var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var sign_exports = {};
__export(sign_exports, {
  FlattenedSign: () => FlattenedSign
});
module.exports = __toCommonJS(sign_exports);
var import_base64url = require("../../runtime/base64url.js");
var import_sign = __toESM(require("../../runtime/sign.js"));
var import_is_disjoint = __toESM(require("../../lib/is_disjoint.js"));
var import_errors = require("../../util/errors.js");
var import_buffer_utils = require("../../lib/buffer_utils.js");
var import_check_key_type = __toESM(require("../../lib/check_key_type.js"));
var import_validate_crit = __toESM(require("../../lib/validate_crit.js"));
class FlattenedSign {
  /** @param payload Binary representation of the payload to sign. */
  constructor(payload) {
    if (!(payload instanceof Uint8Array)) {
      throw new TypeError("payload must be an instance of Uint8Array");
    }
    this._payload = payload;
  }
  /**
   * Sets the JWS Protected Header on the FlattenedSign object.
   *
   * @param protectedHeader JWS Protected Header.
   */
  setProtectedHeader(protectedHeader) {
    if (this._protectedHeader) {
      throw new TypeError("setProtectedHeader can only be called once");
    }
    this._protectedHeader = protectedHeader;
    return this;
  }
  /**
   * Sets the JWS Unprotected Header on the FlattenedSign object.
   *
   * @param unprotectedHeader JWS Unprotected Header.
   */
  setUnprotectedHeader(unprotectedHeader) {
    if (this._unprotectedHeader) {
      throw new TypeError("setUnprotectedHeader can only be called once");
    }
    this._unprotectedHeader = unprotectedHeader;
    return this;
  }
  /**
   * Signs and resolves the value of the Flattened JWS object.
   *
   * @param key Private Key or Secret to sign the JWS with.
   * @param options JWS Sign options.
   */
  async sign(key, options) {
    if (!this._protectedHeader && !this._unprotectedHeader) {
      throw new import_errors.JWSInvalid(
        "either setProtectedHeader or setUnprotectedHeader must be called before #sign()"
      );
    }
    if (!(0, import_is_disjoint.default)(this._protectedHeader, this._unprotectedHeader)) {
      throw new import_errors.JWSInvalid(
        "JWS Protected and JWS Unprotected Header Parameter names must be disjoint"
      );
    }
    const joseHeader = {
      ...this._protectedHeader,
      ...this._unprotectedHeader
    };
    const extensions = (0, import_validate_crit.default)(
      import_errors.JWSInvalid,
      /* @__PURE__ */ new Map([["b64", true]]),
      options == null ? void 0 : options.crit,
      this._protectedHeader,
      joseHeader
    );
    let b64 = true;
    if (extensions.has("b64")) {
      b64 = this._protectedHeader.b64;
      if (typeof b64 !== "boolean") {
        throw new import_errors.JWSInvalid(
          'The "b64" (base64url-encode payload) Header Parameter must be a boolean'
        );
      }
    }
    const { alg } = joseHeader;
    if (typeof alg !== "string" || !alg) {
      throw new import_errors.JWSInvalid('JWS "alg" (Algorithm) Header Parameter missing or invalid');
    }
    (0, import_check_key_type.default)(alg, key, "sign");
    let payload = this._payload;
    if (b64) {
      payload = import_buffer_utils.encoder.encode((0, import_base64url.encode)(payload));
    }
    let protectedHeader;
    if (this._protectedHeader) {
      protectedHeader = import_buffer_utils.encoder.encode((0, import_base64url.encode)(JSON.stringify(this._protectedHeader)));
    } else {
      protectedHeader = import_buffer_utils.encoder.encode("");
    }
    const data = (0, import_buffer_utils.concat)(protectedHeader, import_buffer_utils.encoder.encode("."), payload);
    const signature = await (0, import_sign.default)(alg, key, data);
    const jws = {
      signature: (0, import_base64url.encode)(signature),
      payload: ""
    };
    if (b64) {
      jws.payload = import_buffer_utils.decoder.decode(payload);
    }
    if (this._unprotectedHeader) {
      jws.header = this._unprotectedHeader;
    }
    if (this._protectedHeader) {
      jws.protected = import_buffer_utils.decoder.decode(protectedHeader);
    }
    return jws;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FlattenedSign
});
