var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var decrypt_key_management_exports = {};
__export(decrypt_key_management_exports, {
  default: () => decrypt_key_management_default
});
module.exports = __toCommonJS(decrypt_key_management_exports);
var import_aeskw = require("../runtime/aeskw.js");
var ECDH = __toESM(require("../runtime/ecdhes.js"));
var import_pbes2kw = require("../runtime/pbes2kw.js");
var import_rsaes = require("../runtime/rsaes.js");
var import_base64url = require("../runtime/base64url.js");
var import_errors = require("../util/errors.js");
var import_cek = require("../lib/cek.js");
var import_import = require("../key/import.js");
var import_check_key_type = __toESM(require("./check_key_type.js"));
var import_is_object = __toESM(require("./is_object.js"));
var import_aesgcmkw = require("./aesgcmkw.js");
async function decryptKeyManagement(alg, key, encryptedKey, joseHeader, options) {
  (0, import_check_key_type.default)(alg, key, "decrypt");
  switch (alg) {
    case "dir": {
      if (encryptedKey !== void 0)
        throw new import_errors.JWEInvalid("Encountered unexpected JWE Encrypted Key");
      return key;
    }
    case "ECDH-ES":
      if (encryptedKey !== void 0)
        throw new import_errors.JWEInvalid("Encountered unexpected JWE Encrypted Key");
    case "ECDH-ES+A128KW":
    case "ECDH-ES+A192KW":
    case "ECDH-ES+A256KW": {
      if (!(0, import_is_object.default)(joseHeader.epk))
        throw new import_errors.JWEInvalid(`JOSE Header "epk" (Ephemeral Public Key) missing or invalid`);
      if (!ECDH.ecdhAllowed(key))
        throw new import_errors.JOSENotSupported(
          "ECDH with the provided key is not allowed or not supported by your javascript runtime"
        );
      const epk = await (0, import_import.importJWK)(joseHeader.epk, alg);
      let partyUInfo;
      let partyVInfo;
      if (joseHeader.apu !== void 0) {
        if (typeof joseHeader.apu !== "string")
          throw new import_errors.JWEInvalid(`JOSE Header "apu" (Agreement PartyUInfo) invalid`);
        partyUInfo = (0, import_base64url.decode)(joseHeader.apu);
      }
      if (joseHeader.apv !== void 0) {
        if (typeof joseHeader.apv !== "string")
          throw new import_errors.JWEInvalid(`JOSE Header "apv" (Agreement PartyVInfo) invalid`);
        partyVInfo = (0, import_base64url.decode)(joseHeader.apv);
      }
      const sharedSecret = await ECDH.deriveKey(
        epk,
        key,
        alg === "ECDH-ES" ? joseHeader.enc : alg,
        alg === "ECDH-ES" ? (0, import_cek.bitLength)(joseHeader.enc) : parseInt(alg.slice(-5, -2), 10),
        partyUInfo,
        partyVInfo
      );
      if (alg === "ECDH-ES")
        return sharedSecret;
      if (encryptedKey === void 0)
        throw new import_errors.JWEInvalid("JWE Encrypted Key missing");
      return (0, import_aeskw.unwrap)(alg.slice(-6), sharedSecret, encryptedKey);
    }
    case "RSA1_5":
    case "RSA-OAEP":
    case "RSA-OAEP-256":
    case "RSA-OAEP-384":
    case "RSA-OAEP-512": {
      if (encryptedKey === void 0)
        throw new import_errors.JWEInvalid("JWE Encrypted Key missing");
      return (0, import_rsaes.decrypt)(alg, key, encryptedKey);
    }
    case "PBES2-HS256+A128KW":
    case "PBES2-HS384+A192KW":
    case "PBES2-HS512+A256KW": {
      if (encryptedKey === void 0)
        throw new import_errors.JWEInvalid("JWE Encrypted Key missing");
      if (typeof joseHeader.p2c !== "number")
        throw new import_errors.JWEInvalid(`JOSE Header "p2c" (PBES2 Count) missing or invalid`);
      const p2cLimit = (options == null ? void 0 : options.maxPBES2Count) || 1e4;
      if (joseHeader.p2c > p2cLimit)
        throw new import_errors.JWEInvalid(`JOSE Header "p2c" (PBES2 Count) out is of acceptable bounds`);
      if (typeof joseHeader.p2s !== "string")
        throw new import_errors.JWEInvalid(`JOSE Header "p2s" (PBES2 Salt) missing or invalid`);
      return (0, import_pbes2kw.decrypt)(alg, key, encryptedKey, joseHeader.p2c, (0, import_base64url.decode)(joseHeader.p2s));
    }
    case "A128KW":
    case "A192KW":
    case "A256KW": {
      if (encryptedKey === void 0)
        throw new import_errors.JWEInvalid("JWE Encrypted Key missing");
      return (0, import_aeskw.unwrap)(alg, key, encryptedKey);
    }
    case "A128GCMKW":
    case "A192GCMKW":
    case "A256GCMKW": {
      if (encryptedKey === void 0)
        throw new import_errors.JWEInvalid("JWE Encrypted Key missing");
      if (typeof joseHeader.iv !== "string")
        throw new import_errors.JWEInvalid(`JOSE Header "iv" (Initialization Vector) missing or invalid`);
      if (typeof joseHeader.tag !== "string")
        throw new import_errors.JWEInvalid(`JOSE Header "tag" (Authentication Tag) missing or invalid`);
      const iv = (0, import_base64url.decode)(joseHeader.iv);
      const tag = (0, import_base64url.decode)(joseHeader.tag);
      return (0, import_aesgcmkw.unwrap)(alg, key, encryptedKey, iv, tag);
    }
    default: {
      throw new import_errors.JOSENotSupported('Invalid or unsupported "alg" (JWE Algorithm) header value');
    }
  }
}
var decrypt_key_management_default = decryptKeyManagement;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
