var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var decrypt_exports = {};
__export(decrypt_exports, {
  default: () => decrypt_default
});
module.exports = __toCommonJS(decrypt_exports);
var import_crypto = require("crypto");
var import_check_iv_length = __toESM(require("../lib/check_iv_length.js"));
var import_check_cek_length = __toESM(require("./check_cek_length.js"));
var import_buffer_utils = require("../lib/buffer_utils.js");
var import_errors = require("../util/errors.js");
var import_timing_safe_equal = __toESM(require("./timing_safe_equal.js"));
var import_cbc_tag = __toESM(require("./cbc_tag.js"));
var import_webcrypto = require("./webcrypto.js");
var import_crypto_key = require("../lib/crypto_key.js");
var import_is_key_object = __toESM(require("./is_key_object.js"));
var import_invalid_key_input = __toESM(require("../lib/invalid_key_input.js"));
var import_ciphers = __toESM(require("./ciphers.js"));
var import_is_key_like = require("./is_key_like.js");
function cbcDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  const keySize = parseInt(enc.slice(1, 4), 10);
  if ((0, import_is_key_object.default)(cek)) {
    cek = cek.export();
  }
  const encKey = cek.subarray(keySize >> 3);
  const macKey = cek.subarray(0, keySize >> 3);
  const macSize = parseInt(enc.slice(-3), 10);
  const algorithm = `aes-${keySize}-cbc`;
  if (!(0, import_ciphers.default)(algorithm)) {
    throw new import_errors.JOSENotSupported(`alg ${enc} is not supported by your javascript runtime`);
  }
  const expectedTag = (0, import_cbc_tag.default)(aad, iv, ciphertext, macSize, macKey, keySize);
  let macCheckPassed;
  try {
    macCheckPassed = (0, import_timing_safe_equal.default)(tag, expectedTag);
  } catch {
  }
  if (!macCheckPassed) {
    throw new import_errors.JWEDecryptionFailed();
  }
  let plaintext;
  try {
    const decipher = (0, import_crypto.createDecipheriv)(algorithm, encKey, iv);
    plaintext = (0, import_buffer_utils.concat)(decipher.update(ciphertext), decipher.final());
  } catch {
  }
  if (!plaintext) {
    throw new import_errors.JWEDecryptionFailed();
  }
  return plaintext;
}
function gcmDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  const keySize = parseInt(enc.slice(1, 4), 10);
  const algorithm = `aes-${keySize}-gcm`;
  if (!(0, import_ciphers.default)(algorithm)) {
    throw new import_errors.JOSENotSupported(`alg ${enc} is not supported by your javascript runtime`);
  }
  try {
    const decipher = (0, import_crypto.createDecipheriv)(algorithm, cek, iv, { authTagLength: 16 });
    decipher.setAuthTag(tag);
    if (aad.byteLength) {
      decipher.setAAD(aad, { plaintextLength: ciphertext.length });
    }
    const plaintext = decipher.update(ciphertext);
    decipher.final();
    return plaintext;
  } catch {
    throw new import_errors.JWEDecryptionFailed();
  }
}
const decrypt = (enc, cek, ciphertext, iv, tag, aad) => {
  let key;
  if ((0, import_webcrypto.isCryptoKey)(cek)) {
    (0, import_crypto_key.checkEncCryptoKey)(cek, enc, "decrypt");
    key = import_crypto.KeyObject.from(cek);
  } else if (cek instanceof Uint8Array || (0, import_is_key_object.default)(cek)) {
    key = cek;
  } else {
    throw new TypeError((0, import_invalid_key_input.default)(cek, ...import_is_key_like.types, "Uint8Array"));
  }
  (0, import_check_cek_length.default)(enc, key);
  (0, import_check_iv_length.default)(enc, iv);
  switch (enc) {
    case "A128CBC-HS256":
    case "A192CBC-HS384":
    case "A256CBC-HS512":
      return cbcDecrypt(enc, key, ciphertext, iv, tag, aad);
    case "A128GCM":
    case "A192GCM":
    case "A256GCM":
      return gcmDecrypt(enc, key, ciphertext, iv, tag, aad);
    default:
      throw new import_errors.JOSENotSupported("Unsupported JWE Content Encryption Algorithm");
  }
};
var decrypt_default = decrypt;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
