var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var encrypt_exports = {};
__export(encrypt_exports, {
  FlattenedEncrypt: () => FlattenedEncrypt,
  unprotected: () => unprotected
});
module.exports = __toCommonJS(encrypt_exports);
var import_base64url = require("../../runtime/base64url.js");
var import_encrypt = __toESM(require("../../runtime/encrypt.js"));
var import_zlib = require("../../runtime/zlib.js");
var import_iv = __toESM(require("../../lib/iv.js"));
var import_encrypt_key_management = __toESM(require("../../lib/encrypt_key_management.js"));
var import_errors = require("../../util/errors.js");
var import_is_disjoint = __toESM(require("../../lib/is_disjoint.js"));
var import_buffer_utils = require("../../lib/buffer_utils.js");
var import_validate_crit = __toESM(require("../../lib/validate_crit.js"));
const unprotected = Symbol();
class FlattenedEncrypt {
  _plaintext;
  _protectedHeader;
  _sharedUnprotectedHeader;
  _unprotectedHeader;
  _aad;
  _cek;
  _iv;
  _keyManagementParameters;
  /** @param plaintext Binary representation of the plaintext to encrypt. */
  constructor(plaintext) {
    if (!(plaintext instanceof Uint8Array)) {
      throw new TypeError("plaintext must be an instance of Uint8Array");
    }
    this._plaintext = plaintext;
  }
  /**
   * Sets the JWE Key Management parameters to be used when encrypting. Use of this is method is
   * really only needed for ECDH based algorithms when utilizing the Agreement PartyUInfo or
   * Agreement PartyVInfo parameters. Other parameters will always be randomly generated when needed
   * and missing.
   *
   * @param parameters JWE Key Management parameters.
   */
  setKeyManagementParameters(parameters) {
    if (this._keyManagementParameters) {
      throw new TypeError("setKeyManagementParameters can only be called once");
    }
    this._keyManagementParameters = parameters;
    return this;
  }
  /**
   * Sets the JWE Protected Header on the FlattenedEncrypt object.
   *
   * @param protectedHeader JWE Protected Header.
   */
  setProtectedHeader(protectedHeader) {
    if (this._protectedHeader) {
      throw new TypeError("setProtectedHeader can only be called once");
    }
    this._protectedHeader = protectedHeader;
    return this;
  }
  /**
   * Sets the JWE Shared Unprotected Header on the FlattenedEncrypt object.
   *
   * @param sharedUnprotectedHeader JWE Shared Unprotected Header.
   */
  setSharedUnprotectedHeader(sharedUnprotectedHeader) {
    if (this._sharedUnprotectedHeader) {
      throw new TypeError("setSharedUnprotectedHeader can only be called once");
    }
    this._sharedUnprotectedHeader = sharedUnprotectedHeader;
    return this;
  }
  /**
   * Sets the JWE Per-Recipient Unprotected Header on the FlattenedEncrypt object.
   *
   * @param unprotectedHeader JWE Per-Recipient Unprotected Header.
   */
  setUnprotectedHeader(unprotectedHeader) {
    if (this._unprotectedHeader) {
      throw new TypeError("setUnprotectedHeader can only be called once");
    }
    this._unprotectedHeader = unprotectedHeader;
    return this;
  }
  /**
   * Sets the Additional Authenticated Data on the FlattenedEncrypt object.
   *
   * @param aad Additional Authenticated Data.
   */
  setAdditionalAuthenticatedData(aad) {
    this._aad = aad;
    return this;
  }
  /**
   * Sets a content encryption key to use, by default a random suitable one is generated for the JWE
   * enc" (Encryption Algorithm) Header Parameter.
   *
   * @deprecated You should not use this method. It is only really intended for test and vector
   *   validation purposes.
   * @param cek JWE Content Encryption Key.
   */
  setContentEncryptionKey(cek) {
    if (this._cek) {
      throw new TypeError("setContentEncryptionKey can only be called once");
    }
    this._cek = cek;
    return this;
  }
  /**
   * Sets the JWE Initialization Vector to use for content encryption, by default a random suitable
   * one is generated for the JWE enc" (Encryption Algorithm) Header Parameter.
   *
   * @deprecated You should not use this method. It is only really intended for test and vector
   *   validation purposes.
   * @param iv JWE Initialization Vector.
   */
  setInitializationVector(iv) {
    if (this._iv) {
      throw new TypeError("setInitializationVector can only be called once");
    }
    this._iv = iv;
    return this;
  }
  /**
   * Encrypts and resolves the value of the Flattened JWE object.
   *
   * @param key Public Key or Secret to encrypt the JWE with.
   * @param options JWE Encryption options.
   */
  async encrypt(key, options) {
    if (!this._protectedHeader && !this._unprotectedHeader && !this._sharedUnprotectedHeader) {
      throw new import_errors.JWEInvalid(
        "either setProtectedHeader, setUnprotectedHeader, or sharedUnprotectedHeader must be called before #encrypt()"
      );
    }
    if (!(0, import_is_disjoint.default)(this._protectedHeader, this._unprotectedHeader, this._sharedUnprotectedHeader)) {
      throw new import_errors.JWEInvalid(
        "JWE Protected, JWE Shared Unprotected and JWE Per-Recipient Header Parameter names must be disjoint"
      );
    }
    const joseHeader = {
      ...this._protectedHeader,
      ...this._unprotectedHeader,
      ...this._sharedUnprotectedHeader
    };
    (0, import_validate_crit.default)(import_errors.JWEInvalid, /* @__PURE__ */ new Map(), options?.crit, this._protectedHeader, joseHeader);
    if (joseHeader.zip !== void 0) {
      if (!this._protectedHeader || !this._protectedHeader.zip) {
        throw new import_errors.JWEInvalid('JWE "zip" (Compression Algorithm) Header MUST be integrity protected');
      }
      if (joseHeader.zip !== "DEF") {
        throw new import_errors.JOSENotSupported(
          'Unsupported JWE "zip" (Compression Algorithm) Header Parameter value'
        );
      }
    }
    const { alg, enc } = joseHeader;
    if (typeof alg !== "string" || !alg) {
      throw new import_errors.JWEInvalid('JWE "alg" (Algorithm) Header Parameter missing or invalid');
    }
    if (typeof enc !== "string" || !enc) {
      throw new import_errors.JWEInvalid('JWE "enc" (Encryption Algorithm) Header Parameter missing or invalid');
    }
    let encryptedKey;
    if (alg === "dir") {
      if (this._cek) {
        throw new TypeError("setContentEncryptionKey cannot be called when using Direct Encryption");
      }
    } else if (alg === "ECDH-ES") {
      if (this._cek) {
        throw new TypeError(
          "setContentEncryptionKey cannot be called when using Direct Key Agreement"
        );
      }
    }
    let cek;
    {
      let parameters;
      ({ cek, encryptedKey, parameters } = await (0, import_encrypt_key_management.default)(
        alg,
        enc,
        key,
        this._cek,
        this._keyManagementParameters
      ));
      if (parameters) {
        if (options && unprotected in options) {
          if (!this._unprotectedHeader) {
            this.setUnprotectedHeader(parameters);
          } else {
            this._unprotectedHeader = { ...this._unprotectedHeader, ...parameters };
          }
        } else {
          if (!this._protectedHeader) {
            this.setProtectedHeader(parameters);
          } else {
            this._protectedHeader = { ...this._protectedHeader, ...parameters };
          }
        }
      }
    }
    this._iv ||= (0, import_iv.default)(enc);
    let additionalData;
    let protectedHeader;
    let aadMember;
    if (this._protectedHeader) {
      protectedHeader = import_buffer_utils.encoder.encode((0, import_base64url.encode)(JSON.stringify(this._protectedHeader)));
    } else {
      protectedHeader = import_buffer_utils.encoder.encode("");
    }
    if (this._aad) {
      aadMember = (0, import_base64url.encode)(this._aad);
      additionalData = (0, import_buffer_utils.concat)(protectedHeader, import_buffer_utils.encoder.encode("."), import_buffer_utils.encoder.encode(aadMember));
    } else {
      additionalData = protectedHeader;
    }
    let ciphertext;
    let tag;
    if (joseHeader.zip === "DEF") {
      const deflated = await (options?.deflateRaw || import_zlib.deflate)(this._plaintext);
      ({ ciphertext, tag } = await (0, import_encrypt.default)(enc, deflated, cek, this._iv, additionalData));
    } else {
      ;
      ({ ciphertext, tag } = await (0, import_encrypt.default)(enc, this._plaintext, cek, this._iv, additionalData));
    }
    const jwe = {
      ciphertext: (0, import_base64url.encode)(ciphertext),
      iv: (0, import_base64url.encode)(this._iv),
      tag: (0, import_base64url.encode)(tag)
    };
    if (encryptedKey) {
      jwe.encrypted_key = (0, import_base64url.encode)(encryptedKey);
    }
    if (aadMember) {
      jwe.aad = aadMember;
    }
    if (this._protectedHeader) {
      jwe.protected = import_buffer_utils.decoder.decode(protectedHeader);
    }
    if (this._sharedUnprotectedHeader) {
      jwe.unprotected = this._sharedUnprotectedHeader;
    }
    if (this._unprotectedHeader) {
      jwe.header = this._unprotectedHeader;
    }
    return jwe;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FlattenedEncrypt,
  unprotected
});
