var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var encrypt_key_management_exports = {};
__export(encrypt_key_management_exports, {
  default: () => encrypt_key_management_default
});
module.exports = __toCommonJS(encrypt_key_management_exports);
var import_aeskw = require("../runtime/aeskw.js");
var ECDH = __toESM(require("../runtime/ecdhes.js"));
var import_pbes2kw = require("../runtime/pbes2kw.js");
var import_rsaes = require("../runtime/rsaes.js");
var import_base64url = require("../runtime/base64url.js");
var import_cek = __toESM(require("../lib/cek.js"));
var import_errors = require("../util/errors.js");
var import_export = require("../key/export.js");
var import_check_key_type = __toESM(require("./check_key_type.js"));
var import_aesgcmkw = require("./aesgcmkw.js");
async function encryptKeyManagement(alg, enc, key, providedCek, providedParameters = {}) {
  let encryptedKey;
  let parameters;
  let cek;
  (0, import_check_key_type.default)(alg, key, "encrypt");
  switch (alg) {
    case "dir": {
      cek = key;
      break;
    }
    case "ECDH-ES":
    case "ECDH-ES+A128KW":
    case "ECDH-ES+A192KW":
    case "ECDH-ES+A256KW": {
      if (!ECDH.ecdhAllowed(key)) {
        throw new import_errors.JOSENotSupported(
          "ECDH with the provided key is not allowed or not supported by your javascript runtime"
        );
      }
      const { apu, apv } = providedParameters;
      let { epk: ephemeralKey } = providedParameters;
      ephemeralKey ||= (await ECDH.generateEpk(key)).privateKey;
      const { x, y, crv, kty } = await (0, import_export.exportJWK)(ephemeralKey);
      const sharedSecret = await ECDH.deriveKey(
        key,
        ephemeralKey,
        alg === "ECDH-ES" ? enc : alg,
        alg === "ECDH-ES" ? (0, import_cek.bitLength)(enc) : parseInt(alg.slice(-5, -2), 10),
        apu,
        apv
      );
      parameters = { epk: { x, crv, kty } };
      if (kty === "EC")
        parameters.epk.y = y;
      if (apu)
        parameters.apu = (0, import_base64url.encode)(apu);
      if (apv)
        parameters.apv = (0, import_base64url.encode)(apv);
      if (alg === "ECDH-ES") {
        cek = sharedSecret;
        break;
      }
      cek = providedCek || (0, import_cek.default)(enc);
      const kwAlg = alg.slice(-6);
      encryptedKey = await (0, import_aeskw.wrap)(kwAlg, sharedSecret, cek);
      break;
    }
    case "RSA1_5":
    case "RSA-OAEP":
    case "RSA-OAEP-256":
    case "RSA-OAEP-384":
    case "RSA-OAEP-512": {
      cek = providedCek || (0, import_cek.default)(enc);
      encryptedKey = await (0, import_rsaes.encrypt)(alg, key, cek);
      break;
    }
    case "PBES2-HS256+A128KW":
    case "PBES2-HS384+A192KW":
    case "PBES2-HS512+A256KW": {
      cek = providedCek || (0, import_cek.default)(enc);
      const { p2c, p2s } = providedParameters;
      ({ encryptedKey, ...parameters } = await (0, import_pbes2kw.encrypt)(alg, key, cek, p2c, p2s));
      break;
    }
    case "A128KW":
    case "A192KW":
    case "A256KW": {
      cek = providedCek || (0, import_cek.default)(enc);
      encryptedKey = await (0, import_aeskw.wrap)(alg, key, cek);
      break;
    }
    case "A128GCMKW":
    case "A192GCMKW":
    case "A256GCMKW": {
      cek = providedCek || (0, import_cek.default)(enc);
      const { iv } = providedParameters;
      ({ encryptedKey, ...parameters } = await (0, import_aesgcmkw.wrap)(alg, key, cek, iv));
      break;
    }
    default: {
      throw new import_errors.JOSENotSupported('Invalid or unsupported "alg" (JWE Algorithm) header value');
    }
  }
  return { cek, encryptedKey, parameters };
}
var encrypt_key_management_default = encryptKeyManagement;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
